%% TransOpt - A codon optimisation tool developed by the Sydney Australia iGEM team of 2015.
%For further explanation, please see http://2015.igem.org/Team:Sydney_Australia/TransOpt

clear
clc
%% Choose Sequence, Start, and Target organisms
start_organism='myco'; %Name of the starting organism's data files.
end_organism='pseudo'; %Name of the end organism's data files.

fname='etnD'; %Name of the txt file to be optimised

%% Read disallowed iGEM sequences
badseq=['GAATTC';'TCTAGA';'ACTAGT';'CTGCAG'];


%% Define optimisation parameters.
RunOptim=0; %Set to 1 to run the optimisation algorithm. Set to 0 if you want to plot previous results.
widthRate=31; %Define the averaging width for rate calculations
widthGC=31; %Define the averaging width for GC Richness
DropGC=1; %Set to 1 to maintain the GC Richness profile. Less than 1 decreases GC content, whilst greater than 1 increases GC content.




%% Now we read in the files about tRNA abundance, and index the different codons and amino acids

% Read the GCN rate data.
[start_gcn_freq codon_names codon_ids]=readOrganism([start_organism,'gcn.csv'],1);
[start_organism_spencer_rate]=SpencerRateCalculate(start_gcn_freq,codon_names);
[end_gcn_freq]=readOrganism([end_organism,'gcn.csv'],0,codon_names);
[end_organism_spencer_rate]=SpencerRateCalculate(end_gcn_freq,codon_names);

outlawed_nums=processseq(badseq,codon_names); %Calculate numeric indeces of disallowed sequences due to iGEM rules.
GC_freq=GC_freq_calc(codon_names); %Calculate GC concentration in each codon
%% Read in the sequence we will be dealing with
    fid=fopen([fname,'.txt']);
    line=fgetl(fid);
    fclose(fid);
    sequence=line;


%% First, convert the sequence of nucleotides to numerical codon / AA numbers
    [sequence_amino,sequence_codon]=decode_sequence(sequence,codon_names,codon_ids);


%% Calculate GC rates
    start_GC_rate=DropGC*SpencerRateImplementB(sequence_codon,GC_freq,widthGC)/3;

%% Calculate Spencer rates for codons in intial / final organisms

[start_spencer_rate]=SpencerRateImplementB(sequence_codon,start_organism_spencer_rate,widthRate); %Calculate specner rates for the two organisms
[end_spencer_rate]=SpencerRateImplementB(sequence_codon,end_organism_spencer_rate,widthRate);

%%  Do standard codon harmonization
    [harmonized_spencer_sequence]=frequency_optimise(sequence_codon,sequence_amino,codon_ids,start_organism_spencer_rate,end_organism_spencer_rate);
    %[harmonized_spencer_sequence]=frequency_optimiseMAX(sequence_codon,sequence_amino,codon_ids,start_organism_spencer_rate,end_organism_spencer_rate);

    %Calculate the rate profile for standard harmonization
    [harmonized_spencer_rate]=SpencerRateImplementB(harmonized_spencer_sequence,end_organism_spencer_rate,widthRate);

%% Do our improved TransOpt method.
    if RunOptim
    trend_optimize_energy(start_spencer_rate,sequence_codon,sequence_amino,codon_ids,start_organism_spencer_rate,end_organism_spencer_rate,widthRate,'sequencebst2.csv',GC_freq,start_GC_rate,outlawed_nums);
    end

    Goodharmonized_spencer_sequence=csvread('sequencebst2.csv'); %Read in the results of the optimisation (if not run in this program run).
    %Calculate the rate profile for TransOpt haromization. 
    harm2=SpencerRateImplementB(Goodharmonized_spencer_sequence,end_organism_spencer_rate,widthRate);

%% Plotting results
    figure(1)
    subplot(2,1,1)
    plot(start_spencer_rate)
    hold on
    plot(harmonized_spencer_rate)
    plot(harm2)
    plot(end_spencer_rate)
    hold off
    legend('Sequence in native host (WT) - target','Sequence in heterologous host with standard codon harmonization','Sequence in heterologous host with TransOpt optimisation','Sequence in heterologous host without adjustment','location','best')
    title('Long-term average translation rate')
    xlabel('mRNA Codon Position, n')
    ylabel('Rate (Arbitrary Units)')

    subplot(2,1,2)
    GC_harm1=SpencerRateImplementB(harmonized_spencer_sequence,GC_freq,widthGC)/3;
    GC_harm2=SpencerRateImplementB(Goodharmonized_spencer_sequence,GC_freq,widthGC)/3;
    plot(start_GC_rate);
    hold on
    plot(GC_harm1)
    plot(GC_harm2)
    hold off
    legend('Sequence in native host (WT) - target','Sequence in heterologous host with standard codon harmonization','Sequence in heterologous host with TransOpt optimisation','location','best')
    title('GC Content')
    ylabel('GC Richness')
    xlabel('mRNA Codon Position, n')



%% Doing a final check and outputting the sequence. 
freq_output=Goodharmonized_spencer_sequence; %Select which sequence will be written to file
[sequence_out]=code_sequence(freq_output,codon_names); %Re-code this sequence into nucleotides

changed=1; %Run a check to remove any iGEM-restricted sites. 
while changed==1
    [freq_output sequence_out changed]=fix_outlawed_final(badseq,freq_output,codon_ids,sequence_out,codon_names);
end

if synonym(sequence_codon,freq_output,codon_ids(:,2))
    disp('Processing has finished, the new sequence is synonymous with the initial sequence.')
end
[sequence_out]=code_sequence(freq_output,codon_names); %%Re-code the sequence
dlmwrite([fname,'STD.txt'],sequence_out,'delimiter', '', 'precision','%c') %Write to file if desired. 
